Simplify the scheduling code.
[paraslash.git] / sched.c
diff --git a/sched.c b/sched.c
index 873fb8e3adca7b1df2d73acd37dbf0764b74e451..9976a11d2aa315f78ff88072c0a27d84ed35e58d 100644 (file)
--- a/sched.c
+++ b/sched.c
@@ -24,19 +24,35 @@ static int initialized;
 static struct timeval now_struct;
 struct timeval *now = &now_struct;
 
+/**
+ * Remove a task from the scheduler.
+ *
+ * \param t the task to remove
+ *
+ * If the pre_select pointer of \a t is not \p NULL, it is removed from
+ * the pre_select list of the scheduler. Same goes for \a post_select.
+ */
+void unregister_task(struct task *t)
+{
+       if (!initialized)
+               return;
+       PARA_INFO_LOG("unregistering %s (%p)\n", t->status, t);
+       if (t->pre_select)
+               list_del(&t->pre_select_node);
+       if (t->post_select)
+               list_del(&t->post_select_node);
+};
+
+
 static void sched_preselect(struct sched *s)
 {
        struct task *t, *tmp;
-again:
        list_for_each_entry_safe(t, tmp, &pre_select_list, pre_select_node) {
                t->pre_select(s, t);
 //             PARA_INFO_LOG("%s \n", t->status);
-               if (t->ret > 0)
+               if (t->error >= 0)
                        continue;
-               if (!t->event_handler)
-                       continue;
-               t->event_handler(t);
-               goto again;
+               unregister_task(t);
        }
 }
 
@@ -47,9 +63,9 @@ static void sched_post_select(struct sched *s)
        list_for_each_entry_safe(t, tmp, &post_select_list, post_select_node) {
                t->post_select(s, t);
 //             PARA_INFO_LOG("%s: %d\n", t->status, t->ret);
-               if (t->ret > 0 || !t->event_handler)
+               if (t->error >= 0)
                        continue;
-               t->event_handler(t);
+               unregister_task(t);
        }
 }
 
@@ -70,6 +86,8 @@ static void sched_post_select(struct sched *s)
  */
 int schedule(struct sched *s)
 {
+       int ret;
+
        if (!initialized)
                return -E_NOT_INITIALIZED;
        gettimeofday(now, NULL);
@@ -79,10 +97,9 @@ again:
        s->timeout = s->default_timeout;
        s->max_fileno = -1;
        sched_preselect(s);
-       s->select_ret = para_select(s->max_fileno + 1, &s->rfds,
-               &s->wfds, &s->timeout);
-       if (s->select_ret < 0)
-               return s->select_ret;
+       ret = para_select(s->max_fileno + 1, &s->rfds, &s->wfds, &s->timeout);
+       if (ret < 0)
+               return ret;
        gettimeofday(now, NULL);
        sched_post_select(s);
        if (list_empty(&pre_select_list) && list_empty(&post_select_list))
@@ -118,33 +135,14 @@ void register_task(struct task *t)
        PARA_INFO_LOG("registering %s (%p)\n", t->status, t);
        if (t->pre_select) {
                PARA_DEBUG_LOG("pre_select: %p\n", &t->pre_select);
-               para_list_add(&t->pre_select_node, &pre_select_list);
+               list_add_tail(&t->pre_select_node, &pre_select_list);
        }
        if (t->post_select) {
                PARA_DEBUG_LOG("post_select: %p\n", &t->pre_select);
-               para_list_add(&t->post_select_node, &post_select_list);
+               list_add_tail(&t->post_select_node, &post_select_list);
        }
 }
 
-/**
- * remove a task from the scheduler
- *
- * \param t the task to remove
- *
- * If the pre_select pointer of \a t is not \p NULL, it is removed from
- * the pre_select list of the scheduler. Same goes for \a post_select.
- */
-void unregister_task(struct task *t)
-{
-       if (!initialized)
-               return;
-       PARA_INFO_LOG("unregistering %s (%p)\n", t->status, t);
-       if (t->pre_select)
-               list_del(&t->pre_select_node);
-       if (t->post_select)
-               list_del(&t->post_select_node);
-};
-
 /**
  * unregister all tasks
  *
@@ -204,8 +202,8 @@ char *get_task_list(void)
  *
  * \param id the task identifier
  *
* Find the task identified by \a id, set the tasks' return value to
* \p -E_TASK_KILLED and call the event handler of the task.
  *
  * \return Positive on success, negative on errors (e.g. if \a id does not
  * correspond to a registered task).
@@ -221,18 +219,16 @@ int kill_task(char *id)
                sprintf(buf, "%p", t);
                if (strcmp(id, buf))
                        continue;
-               t->ret = -E_TASK_KILLED;
-               if (t->event_handler)
-                       t->event_handler(t);
+               t->error = -E_TASK_KILLED;
+               unregister_task(t);
                return 1;
        }
        list_for_each_entry_safe(t, tmp, &post_select_list, post_select_node) {
                sprintf(buf, "%p", t);
                if (strcmp(id, buf))
                        continue;
-               t->ret = -E_TASK_KILLED;
-               if (t->event_handler)
-                       t->event_handler(t);
+               t->error = -E_TASK_KILLED;
+               unregister_task(t);
                return 1;
        }
        return -E_NO_SUCH_TASK;
@@ -204,8 +202,8 @@ char *get_task_list(void)
  * \param id the task identifier
  *
- * Find the task identified by \a id, set the tasks' return value to
- * \p -E_TASK_KILLED and call the event handler of the task.
+ * Find the task identified by \a id, set the tasks' error value to
+ * \p -E_TASK_KILLED and unregister the task.
  *
  * \return Positive on success, negative on errors (e.g. if \a id does not